============
Contributing
============
When contributing to this repository, please first discuss the change you wish to make via an issue to the owners of this repository before submitting a pull request.

.. important:: **We have an enforced style guide and a code of conduct.**
   Please follow them in all your interactions with this project.


Style Guide
-----------

- We somewhat follow `PEP8 <https://www.python.org/dev/peps/pep-0008/>`_ and utilize `Sphinx <http://www.sphinx-doc.org/en/stable/>`_ docstrings on **all** classes and functions.
- We employee `flake8 <http://flake8.pycqa.org/en/latest/>`_ as our linter with exceptions to the following rules:
   - D203
   - F401
   - E123
   - W503
   - E203
- Maximum line length for Python files is 88 characters.
- Maxiumum McCabe complexity is 13.
- Linting and test environments are configured via ``tox.ini``.
- Imports are sorted using `isort <https://pypi.python.org/pypi/isort>`_ with multi-line output mode ``3``.
- An ``.editorconfig`` file is included in this repository which dictates whitespace, indentation, and file encoding rules.
- Although ``requirements.txt`` and ``requirements-dev.txt`` do exist, `Pipenv <https://docs.pipenv.org/>`_ is utilized as the primary virtual environment and package manager for this project.
- We strictly utilize `Semantic Versioning <https://semver.org/>`_ as our version specification.
- All Python source files are post-processed using `ambv/black <https://github.com/ambv/black>`_.

Issues
------
Issues should follow the included ``ISSUE_TEMPLATE`` found in ``.github/ISSUE_TEMPLATE.md``.

- Issues should contain the following sections:
   - Expected Behavior
   - Current Behavior
   - Possible Solution
   - Steps to Reproduce (for bugs)
   - Context
   - Your Environment

These sections help the developers greatly by providing a large understanding of the context of the bug or requested feature without having to launch a full fleged discussion inside of the issue.

Pull Requests
-------------
Pull requests should follow the included ``PULL_REQUEST_TEMPLATE`` found in ``.github/PULL_REQUEST_TEMPLATE.md``.

- Pull requests should always be from a **topic/feature/bugfix** (left side) branch. *Pull requests from master branches will not be merged.*
- Pull requests should not fail our requested style guidelines or linting checks.

Code of Conduct
---------------
Our code of conduct is taken directly from the `Contributor Covenant <https://www.contributor-covenant.org/>`_ since it directly hits all of the points we find necessary to address.

Our Pledge
''''''''''
In the interest of fostering an open and welcoming environment, we as contributors and maintainers pledge to making participation in our project and our community a harassment-free experience for everyone, regardless of age, body size, disability, ethnicity, gender identity and expression, level of experience, education, socio-economic status, nationality, personal appearance, race, religion, or sexual identity and orientation.

Our Standards
'''''''''''''
Examples of behavior that contributes to creating a positive environment include:

- Using welcoming and inclusive language
- Being respectful of differing viewpoints and experiences
- Gracefully accepting constructive criticism
- Focusing on what is best for the community
- Showing empathy towards other community members

Examples of unacceptable behavior by participants include:

- The use of sexualized language or imagery and unwelcome sexual attention or advances
- Trolling, insulting/derogatory comments, and personal or political attacks
- Public or private harassment
- Publishing others’ private information, such as a physical or electronic address, without explicit permission
- Other conduct which could reasonably be considered inappropriate in a professional setting

Our Responsibilities
''''''''''''''''''''
Project maintainers are responsible for clarifying the standards of acceptable behavior and are expected to take appropriate and fair corrective action in response to any instances of unacceptable behavior.

Project maintainers have the right and responsibility to remove, edit, or reject comments, commits, code, wiki edits, issues, and other contributions that are not aligned to this Code of Conduct, or to ban temporarily or permanently any contributor for other behaviors that they deem inappropriate, threatening, offensive, or harmful.

Scope
'''''
This Code of Conduct applies both within project spaces and in public spaces when an individual is representing the project or its community. Examples of representing a project or community include using an official project e-mail address, posting via an official social media account, or acting as an appointed representative at an online or offline event. Representation of a project may be further defined and clarified by project maintainers.

Enforcement
'''''''''''
Instances of abusive, harassing, or otherwise unacceptable behavior may be reported by contacting the project team at stephen@bunn.io. All complaints will be reviewed and investigated and will result in a response that is deemed necessary and appropriate to the circumstances. The project team is obligated to maintain confidentiality with regard to the reporter of an incident. Further details of specific enforcement policies may be posted separately.

Project maintainers who do not follow or enforce the Code of Conduct in good faith may face temporary or permanent repercussions as determined by other members of the project’s leadership.

Attribution
'''''''''''
This Code of Conduct is adapted from the `Contributor Covenant <https://www.contributor-covenant.org/>`_, version 1.4, available at https://www.contributor-covenant.org/version/1/4/code-of-conduct.html

